from minio import Minio
from pathlib import Path
from glob import glob
from io import BytesIO
from PIL import Image

from .meta import Metadata, create_mongo_filter


class MinioClient(Minio):
    def __init__(self, access_key: str, secret_key: str, endpoint: str):
        super().__init__(
            endpoint,
            access_key=access_key,
            secret_key=secret_key,
            secure=True
        )

    def upload_file(self, bucket_name: str, file_path: str, metadata: Metadata):
        file_name = Path(file_path).name
        self.fput_object(bucket_name=bucket_name, object_name=file_name, file_path=file_path, metadata=metadata.dict())

    def upload_files(self, bucket_name: str, dir_path: str, metadata: Metadata):
        for file in glob(dir_path):
            self.upload_file(bucket_name, file, metadata)

    def upload_binary_file(self, bucket_name: str, file_path: str, metadata: Metadata):
        with open(file_path, 'rb') as f:
            data = f.read()
        # self.put_object(bucket_name, Path(file_path).name, BytesIO(data), length=len(data), metadata=metadata.dict())
        self.upload_binary(bucket_name, Path(file_path).name, BytesIO(data), len(data), metadata)

    def upload_binary(self, bucket_name: str, object_name: str, data: BytesIO, length: int, metadata: Metadata):
        self.put_object(bucket_name, object_name, data, length=length, metadata=metadata.dict())

    def get_all_images_by_example(self, obj, collection):
        """Returns a mongo query iterable object"""

        for item in collection.find(create_mongo_filter(obj)):
            # TODO check if that is an image
            image_hex_bytes = self.get_object(item["event"]["bucket"]["name"], item["event"]["object"]["key"])
            image_stream = BytesIO(image_hex_bytes.read())
            image_file = Image.open(image_stream)
            yield image_file, item



from typing import Optional
from pydantic import BaseModel


class Object(BaseModel):
    key: str
    size: int
    eTag: str
    contentType: str

    @staticmethod
    def load(object_obj):
        return Object.parse_obj({
            'key': object_obj['key'],
            'size': object_obj['size'],
            'eTag': object_obj['eTag'],
            'contentType': object_obj['contentType']
        })


class Bucket(BaseModel):
    name: str
    owner: str

    @staticmethod
    def load(bucket_obj):
        return Bucket.parse_obj({
            'name': bucket_obj['name'],
            'owner': bucket_obj['ownerIdentity']['principalId']
        })


class GpsData(BaseModel):
    x: float = 0.0
    y: float = 0.0
    yaw: Optional[float] = 0.0
    fix: int = 0
    velocity: Optional[float] = 0.0


class IdData(BaseModel):
    location: str
    device: str
    uploader: Optional[str]
    project: Optional[str]
    time: str
    gps: Optional[GpsData]


class EventData(BaseModel):
    type: str = ''
    time: str = ''
    sourceIp: str = ''
    bucket: Bucket = None
    object: Object = None

    @staticmethod
    def load(event_obj: dict):
        event_obj = event_obj['Records'][0]
        return EventData.parse_obj({
            'type': event_obj['eventName'],
            'time': event_obj['eventTime'],
            'sourceIp': event_obj['requestParameters']['sourceIPAddress'],
            'bucket': Bucket.load(event_obj['s3']['bucket']),
            'object': Object.load(event_obj['s3']['object'])
        })


class Metadata(BaseModel):
    id: IdData
    custom: Optional[dict]

    @staticmethod
    def load(id_data: IdData, custom_data: dict = None):
        if not custom_data:
            custom_data = {}

        return Metadata.parse_obj({
            'id': id_data,
            'custom': custom_data
        })


class MongoData(BaseModel):
    event: EventData
    meta: Metadata

    @staticmethod
    def load(event_data: EventData, meta_data: Metadata):
        return MongoData.parse_obj({
            'event': event_data,
            'meta': meta_data
        })


def create_mongo_filter(metadata_obj: Metadata = None, location=None, device=None, gps=None, time=None, source_ip=None,
                        bucket_name=None, file_name=None, **custom_metadata):
    """Returns a dictionary in mongo format

    Use the return dict as paramenter in mongo query funcitons such as find()

    Any other parameter passed will be interpreted as custom
    """

    if metadata_obj is not None:
        # As the user filled the metadata_obj, no other values must be filled in
        for parameter, value in locals().items():
            assert value is None or parameter in ["metadata_obj", "custom_metadata"], \
                "If you provide metadata_obj, leave any other parameter blank"
        assert custom_metadata == {}

        # Uncomment if you want to accept both metadata and metadata.dict() parameters
        # Make sure to change every metadata.x to metadata["x"] down here
        # if not isinstance(metadata_obj, dict):
        #     metadata_obj = metadata_obj.dict()
        location = metadata_obj.id.location
        device = metadata_obj.id.device
        gps = metadata_obj.id.gps
        time = metadata_obj.id.time
        # source_ip, bucket_name, file_name are in "event", not here...
        custom_metadata = metadata_obj.custom

    res = {
        "meta.id.location": location,
        "meta.id.device": device,
        "meta.id.gps": gps,
        "meta.id.time": time,
        "event.sourceIp": source_ip,
        "event.bucket.name": bucket_name,
        "event.object.key": file_name,
    }

    for k, v in custom_metadata.items():
        res["meta.custom." + k] = v

    for k, v in list(res.items()):
        if v is None:
            del res[k]

    return res



from setuptools import setup, find_packages
import codecs
import os

VERSION = "0.0.3"
DESCRIPTION = "Python API to upload data to ILVO's minio server"

setup(
    name="minilvo",
    version=VERSION,
    author="Lorenzo Mogicato",
    author_email="<lorenzo.mogicato@gmail.com>",
    description=DESCRIPTION,
    long_description_content_type="text/markdown",
    packages=find_packages(),
    install_requires=[],  # TODO insert dependencies
    # keywords=['tag1test', 'lollo', '3rdtag'],
    # classifiers=[
    #     "Development Status :: 1 - Planning",
    #     # "Intended Audience :: ILVOTeam",
    #     "Programming Language :: Python :: 3",
    #     "Operating System :: Unix",
    #     "Operating System :: MacOS :: MacOS X",
    #     "Operating System :: Microsoft :: Windows",
    # ]
)
